var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head><script src=\"../../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"></script><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Working with the scene graph</title>\n\
   <meta name=\"topic-subtype\" content=\"Python\"></head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style><script>$(document).ready(function() { yepnope.injectJs(\"./scripts/multireflink.js\"); });</script><script></script><div id=\"reflinkdiv\"></div>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Working with the scene graph</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"working-with-the-scene-graph\"></a></div>\n\
<div class=\'section\'><a id=\"scene-graph-and-nodes\"></a><h2 id=\"scene-graph-and-nodes\">Scene graph and nodes</h2></div>\n\
<p>A scene graph is a tree of scene nodes. Each node is a hub of information for objects in the scene, and it can be responsible for the following:</p>\n\
<ul>\n\
<li>applying modifiers to an object: <span class=\'code\'><a href=\"javascript:void(0)\" data-symbol=\"INode.AddModifier()\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;py_ref/class_max_plus_1_1_i_node.html#a46a871678b7f1ef11bcac01c025229bc&quot;,&quot;title&quot;:&quot;MaxPlus Python API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">INode.AddModifier()</a></span></li>\n\
<li>storing information about the geometry: <span class=\'code\'>INode.BaseObject.ParameterBlock</span></li>\n\
<li>storing information about the shading material that is applied to the object: <span class=\'code\'>INode.Material</span></li>\n\
<li>storing the position, rotation and scaling information of an object: <span class=\'code\'>INode.Scaling</span>, <span class=\'code\'>INode.Position</span>, <span class=\'code\'>INode.Rotation</span></li>\n\
<li>storing node tree hierarchical information such as parent/child relationships: <span class=\'code\'>INode.Parent</span>, <span class=\'code\'>INode.Children</span></li>\n\
</ul>\n\
<div class=\'section\'><a id=\"traversing-the-scene-graph\"></a><h2 id=\"traversing-the-scene-graph\">Traversing the scene graph</h2></div>\n\
<p><span class=\'code\'><a href=\"javascript:void(0)\" data-symbol=\"Core.GetRootNode()\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;py_ref/class_max_plus_1_1_core.html#a41484fba15d54f0b8be71e0fc0b836c8&quot;,&quot;title&quot;:&quot;MaxPlus Python API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">Core.GetRootNode()</a></span> provides the root node of the scene. For each <span class=\'code\'><a href=\"javascript:void(0)\" data-symbol=\"INode\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;py_ref/class_max_plus_1_1_i_node.html&quot;,&quot;title&quot;:&quot;MaxPlus Python API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">INode</a></span>, you can obtain its <span class=\'code\'>Name</span> and <span class=\'code\'>Children</span> via the <span class=\'code\'><a href=\"javascript:void(0)\" data-symbol=\"INode\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;py_ref/class_max_plus_1_1_i_node.html&quot;,&quot;title&quot;:&quot;MaxPlus Python API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">INode</a></span> properties. The example below (<span class=\'code\'><a href=\"javascript:void(0)\" data-symbol=\"demoSceneGraph.py\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;py_ref/demo_scene_graph_8py-example.html&quot;,&quot;title&quot;:&quot;MaxPlus Python API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">demoSceneGraph.py</a></span>) demonstrates how to traverse through the scene graph and print the name of each node and that of its children.</p>\n\
<div class=\'codeBlock\'><pre>&#39;&#39;&#39;\n\
    Creates a simple text representation of the scene graph\n\
&#39;&#39;&#39;\n\
import MaxPlus\n\
import sys\n\
\n\
def outputNode(n, indent = &#39;&#39;):\n\
    print indent, n.Name\n\
    for c in n.Children:\n\
        outputNode(c, indent + &#39;--&#39;)\n\
        \n\
if __name__ == &#39;__main__&#39;:\n\
    outputNode(<a href=\"javascript:void(0)\" data-symbol=\"MaxPlus.Core.GetRootNode\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;py_ref/class_max_plus_1_1_core.html#a41484fba15d54f0b8be71e0fc0b836c8&quot;,&quot;title&quot;:&quot;MaxPlus Python API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">MaxPlus.Core.GetRootNode</a>())\n\
</pre></div><div class=\'section\'><a id=\"selection-list\"></a><h3 id=\"selection-list\">Selection list</h3></div>\n\
<p>While traversing the scene graph, you may find it useful to obtain the list of selected objects.</p>\n\
<p><span class=\'code\'>SelectionManager.Nodes</span> provides an iterator for the selection list.</p>\n\
<p>You can now do the following:</p>\n\
<div class=\'codeBlock\'><pre>for n in MaxPlus.SelectionManager.Nodes:\n\
    doThisWithNode(n)\n\
</pre></div><p>This method takes a material m as input and assigns the material to only the selected objects:</p>\n\
<div class=\'codeBlock\'><pre>def applyMaterialToNodes(m):\n\
    for n in MaxPlus.SelectionManager.Nodes:\n\
        n.Material = m\n\
</pre></div><p>However, because this is an iterator, you cannot directly write to its indexed elements. For example, you cannot write MaxPlus.SelectionManager.Nodes[3]. You must first create a copy of the selection list by converting the iterator to an indexable collection, such as <span class=\'code\'>list(MaxPlus.SelectionManager.Nodes)</span> or <span class=\'code\'>tuple(MaxPlus.SelectionManager.Nodes)</span>. Each element in the list is now a handle to the actual node.</p>\n\
<div class=\'section\'><a id=\"querying-the-types-of-objects-that-exist-in-the-scene\"></a><h2 id=\"querying-the-types-of-objects-that-exist-in-the-scene\">Querying the types of objects that exist in the scene</h2></div>\n\
<p>The example <span class=\'code\'><a href=\"javascript:void(0)\" data-symbol=\"demoTypeCasting.py\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;py_ref/demo_type_casting_8py-example.html&quot;,&quot;title&quot;:&quot;MaxPlus Python API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">demoTypeCasting.py</a></span> demonstrates how to iterate through the scene and print out what class each object belongs to. For each node in the scene, it first obtains the super class ID for each object, then casts it to the correct type and prints it out.</p>\n\
      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";