var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head><script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"></script><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Troubleshooting 3ds Max Python API</title>\n\
   <meta name=\"topic-subtype\" content=\"Python\"></head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style><script>$(document).ready(function() { yepnope.injectJs(\"./scripts/multireflink.js\"); });</script><script></script><div id=\"reflinkdiv\"></div>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Troubleshooting 3ds Max Python API</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"troubleshooting-3ds-max-python-api\"></a></div>\n\
<div class=\'section\'><a id=\"why-doesnt-my-cpython-extension-module-load\"></a><h2 id=\"why-doesnt-my-cpython-extension-module-load\">Why doesn&#39;t my CPython extension module load?</h2></div>\n\
<p>Check if your extension module was built and linked against a different version of Python. In order to use a CPython module with Python in 3ds Max, you must compile the CPython binaries against Python version 2.7.15. See <a href=\"#!/url=./developer/python_extension_libraries.html\">Python extension libraries</a> for more information.</p>\n\
<div class=\'section\'><a id=\"i-am-trying-to-install-an-extension-module-but-3ds-max-python-is-not-detected-by-the-installer\"></a><h2 id=\"i-am-trying-to-install-an-extension-module-but-3ds-max-python-is-not-detected-by-the-installer\">I am trying to install an extension module but 3ds Max Python is not detected by the installer</h2></div>\n\
<p>Update the registry or manually install the module. Often modules support installation via manual steps or a Python module (for example, the Python &quot;easy_install&quot; module (<a href=\'http://pythonhosted.org/distribute/easy_install.html\' title=\'\' target=\'_blank\'>http://pythonhosted.org/distribute/easy_install.html</a>). See also <a href=\"#!/url=./developer/python_extension_libraries.html\">Python extension libraries</a> for more information.</p>\n\
<div class=\'section\'><a id=\"i-set-a-property-but-do-not-see-the-expected-result\"></a><h2 id=\"i-set-a-property-but-do-not-see-the-expected-result\">I set a property but do not see the expected result</h2></div>\n\
<p>In Python, if you misspell a property when assigning it, a new field is added to the object as a result - for example: MyClass.fOotball = x, or worse yet: MyClass.GetValue = 42. It is legal to add or replace fields on an object at run-time; therefore no error occurs. The safest way to set a property is to use a <span class=\'code\'>Set</span> function, for example: render.SetWidth(640).</p>\n\
<div class=\'section\'><a id=\"i-run-my-script-and-it-works.-however-when-i-restart-3ds-max-and-run-the-script-again-i-get-an-error.\"></a><h2 id=\"i-run-my-script-and-it-works.-however-when-i-restart-3ds-max-and-run-the-script-again-i-get-an-error.\">I run my script and it works. However, when I restart 3ds Max and run the script again, I get an error.</h2></div>\n\
<p>When you run any Python code that affects the global environment (for example, importing a module), those settings persist during your session of 3ds Max. A common mistake is to rely on a module that was imported by another script, or to call a function that was created by another script.</p>\n\
<div class=\'section\'><a id=\"i-have-a-script-for-example-mylibrarymodule-that-is-imported-by-another-script-for-example-myprogram-but-after-i-make-changes-to-mylibrarymodule-the-changes-do-not-seem-to-take-effect-the-next-time-i-run-myprogram.\"></a><h2 id=\"i-have-a-script-for-example-mylibrarymodule-that-is-imported-by-another-script-for-example-myprogram-but-after-i-make-changes-to-mylibrarymodule-the-changes-do-not-seem-to-take-effect-the-next-time-i-run-myprogram.\">I have a script (for example, &quot;MyLibraryModule&quot;) that is imported by another script (for example, &quot;MyProgram), but after I make changes to MyLibraryModule, the changes do not seem to take effect the next time I run &quot;MyProgram&quot;.</h2></div>\n\
<p>Use the <span class=\'code\'>reload(MyLibraryModule)</span> command to force Python to reload that module.</p>\n\
<div class=\'section\'><a id=\"common-issues-and-their-workarounds\"></a><h2 id=\"common-issues-and-their-workarounds\">Common issues and their workarounds</h2></div>\n\
<ul>\n\
<li><p>When <span class=\'code\'>sys.exit()</span> is used to exit Python, it raises a <span class=\'code\'>SystemExit</span> exception. If an uncaught exception is raised in Python, it is reported to the MAXScript Listener window as an error, even though no error actually occurred. To avoid having an error being reported in the MAXScript Listener window by <span class=\'code\'>sys.exit()</span>, you can use the <span class=\'code\'>try</span> and <span class=\'code\'>except</span> statements as follows: try: import sys print &#39;goodbye world&#39; sys.exit() # Safe way to leave program at any point and allow objects and resources to be cleaned up. # Prevent 3ds Max from reporting a system exit as an error in the listener. except SystemExit: pass</p></li>\n\
<li>If you import a module in a script, the module remains &quot;imported&quot; even after you finish executing your script and start executing a different script. Any names added to the global namespace will also remain from script execution to script execution. These only persist per session of 3ds Max, however, and you can workaround this by exiting and reloading 3ds Max.</li>\n\
<li><p>When creating an instance of <span class=\'code\'>QApplication()</span>, do not use <span class=\'code\'>app = QApplication([])</span>, use instead:</p>\n\
<div class=\'codeBlock\'><pre>app = PySide2.QtCore.QCoreApplication.instance()\n\
if not app:\n\
    ...\n\
</pre></div><p>to avoid a potential crash.</p>\n\
<p><strong>Note</strong> Troubleshooting tips for PySide2 also apply to PyQt.</p></li>\n\
<li>Some modules require that you set <span class=\'code\'>sys.argv = [&#39;&#39;]</span> before you use them; for example, <span class=\'code\'>app = QtGui.QApplication(sys.argv)</span>.</li>\n\
<li>Module names (for example, <span class=\'code\'><a href=\"javascript:void(0)\" data-symbol=\"MaxPlus\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;py_ref/namespace_max_plus.html&quot;,&quot;title&quot;:&quot;MaxPlus Python API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../&quot;}]\">MaxPlus</a></span>) and built-in functions (for example, <span class=\'code\'>dir</span>) can be re-assigned to anything. This is especially a problem with global variables since it can break other scripts, but is acceptable within the scope of a function.</li>\n\
<li><p>In order to capture keyboard input for your Qt windows, you must disable the 3ds Max keyboard accelerators by calling the following when your UI control gets focus:</p>\n\
<div class=\'codeBlock\'><pre><a href=\"javascript:void(0)\" data-symbol=\"MaxPlus.CUI.DisableAccelerators\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;py_ref/class_max_plus_1_1_c_u_i.html#a3061dac3a3cfaff04a9c06fce458bc1d&quot;,&quot;title&quot;:&quot;MaxPlus Python API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../&quot;}]\">MaxPlus.CUI.DisableAccelerators</a>()\n\
</pre></div><p>You can optionally call the following when your UI control loses focus as well:</p>\n\
<div class=\'codeBlock\'><pre><a href=\"javascript:void(0)\" data-symbol=\"MaxPlus.CUI.EnableAccelerators\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;py_ref/class_max_plus_1_1_c_u_i.html#a55590cdc3eccf1c608d55e17201c2938&quot;,&quot;title&quot;:&quot;MaxPlus Python API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../&quot;}]\">MaxPlus.CUI.EnableAccelerators</a>()\n\
</pre></div><p>There are several areas of 3ds Max that calls <span class=\'code\'>enableAccelerators</span>. For more information, see the <em>MAXScript Documentation</em>.</p></li>\n\
<li><p>The <span class=\'code\'>sys.path</span> does not contain the executing script path. This is by design. To get the path of the currently executing script, use the <span class=\'code\'>__file__</span> constant. Note that this variable is only populated when scripts are run from the <strong>Scripting &gt; Run Script</strong> menu, and is empty when scripts are run from the MAXScript editor.</p></li>\n\
</ul>\n\
      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";