var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head><script src=\"../../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"></script><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Controller Access With pymxs</title>\n\
   <meta name=\"topic-subtype\" content=\"Python\"></head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style><script>$(document).ready(function() { yepnope.injectJs(\"./scripts/multireflink.js\"); });</script><script></script><div id=\"reflinkdiv\"></div>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Controller Access With pymxs</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"controller-access-with-pymxs\"></a></div>\n\
<p>Working with controllers in pymxs requires a different approach than in MAXScript. This is because MAXScript &quot;knows&quot; about nested object properties, and does some extra parsing when it encounters them. The pymxs layer doesn&#39;t understand nested object properties, so these properties do not appear to exist.</p>\n\
<p>Here is an example of some MAXScript code:</p>\n\
<div class=\'codeBlock\'><pre>myTeapot = teapot()\n\
myTeapot.pos.controller = noise_position()\n\
</pre></div><p>In MAXScript, as in Python, <span class=\'code\'>myTeapot.pos</span> is a Point3, which doesn&#39;t have an associated controller property. Attempting to get <span class=\'code\'>myTeapot.pos.controller</span> in Python results in an error such as: <span class=\'code\'>AttributeError: &#39;pymxs.MXSWrapperBase&#39; object has no attribute &#39;controller&#39;</span>.</p>\n\
<p>But in MAXScript <span class=\'code\'>myTeapot.pos.controller</span> is perfectly valid, as it evaluates to an object structure hierarchy. Therefore, to achieve the same thing using pymxs, we must use the same approach and use <span class=\'code\'>getPropertyController()</span> / <span class=\'code\'>setPropertyController()</span>.</p>\n\
<blockquote>\n\
<p><strong>Note:</strong> In general you can use the same syntax as MAXScript to access properties, but in situations where the property is nested and inaccessible to Python/pymxs, you can use <span class=\'code\'>getProperty()</span> / <span class=\'code\'>setProperty()</span>. See the topic <strong>Class and Object Inspector Functions</strong> in the <em>MAXScript Help</em> for more information about these functions.</p>\n\
</blockquote>\n\
<p>Here&#39;s a complete example where we set the Position XYZ controller of a box, and the X component of the position for a sphere:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">from pymxs import runtime as rt\n\
\n\
# set the transform -&gt; position controller\n\
t = rt.teapot()\n\
rt.setPropertyController( t.controller, &#39;Position&#39;, rt.noise_position())\n\
print rt.getPropertyController(t.controller, &#39;Position&#39;)\n\
\n\
# set the transform -&gt; position -&gt; x_position controller\n\
s = rt.sphere()\n\
sphr_pos_ctrl = rt.getPropertyController(s.controller, &#39;Position&#39;)\n\
rt.setPropertyController(sphr_pos_ctrl, &#39;X Position&#39;, rt.noise_float())\n\
print rt.getPropertyController(sphr_pos_ctrl, &#39;X Position&#39;)\n\
</pre></div><p>Output:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">Controller:Noise_Position\n\
Controller:Noise_Float\n\
</pre></div>      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";