var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head><script src=\"../../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"></script><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Tips and Tricks for using pymxs</title>\n\
   <meta name=\"topic-subtype\" content=\"Python\"></head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style><script>$(document).ready(function() { yepnope.injectJs(\"./scripts/multireflink.js\"); });</script><script></script><div id=\"reflinkdiv\"></div>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Tips and Tricks for using pymxs</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"tips-and-tricks-for-using-pymxs\"></a></div>\n\
<div class=\'section\'><a id=\"gizmos-in-pymxs\"></a><h2 id=\"gizmos-in-pymxs\">Gizmos in pymxs</h2></div>\n\
<p>Accessing the gizmo of a modifier (for those modifiers that have a gizmo) can be tricky. Consider this code:</p>\n\
<div class=\'codeBlock\'><pre>myMod = rt.UVWMap()\n\
rt.addModifier(myTeapot, myMod)\n\
myMod.gizmo.position = rt.Point3(20,20,20)\n\
</pre></div><p>Although this works in MAXScript, in Python it produces the error: <span class=\'code\'>AttributeError: &#39;pymxs.MXSWrapperBase&#39; object has no attribute &#39;gizmo&#39;</span>.</p>\n\
<p>The reason is that the Python script does not force a scene graph evaluation the way that MAXScript does, so the gizmo is not created on the node by the time the script tries to access it. These situations can be worked around by forcing a viewport redraw, as in the following modified script:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">from pymxs import runtime as rt\n\
\n\
myTeapot = rt.teapot()\n\
rt.convertTo(myTeapot, rt.editable_poly)\n\
myMod = rt.UVWMap()\n\
rt.addModifier(myTeapot, myMod)\n\
rt.redrawViews() # needed to make the gizmo available\n\
myMod.gizmo.position = rt.Point3(20,20,20)\n\
print myMod.gizmo.position\n\
</pre></div><div class=\'section\'><a id=\"non-callable-maxscript-constructs\"></a><h2 id=\"non-callable-maxscript-constructs\">Non-callable MAXScript Constructs</h2></div>\n\
<p>Although pymxs supports conversion between many Python and MAXScript types, and most &quot;non-Pythonic&quot; constructs (such as passing arguments by reference), you may find some MAXScript commands are simply impossible to replicate in Python. For example, using the &quot;as&quot; coercion syntax. In MAXScript you can do something like this:</p>\n\
<div class=\'codeBlock\'><pre>selectedFaces = getFaceSelection selection[1] -- returns BitArray value\n\
myArray = selectedFaces as Array\n\
</pre></div><p>In Python this is not possible. Instead, we can create a function in MAXScript that does this coercion internally and returns the result. Global functions will be visible to the MAXScript system for the lifetime of the 3ds Max instance. You can do this in a separate MAXScript script, or within your Python script using <span class=\'code\'>MaxPlus.Core.EvalMAXScript()</span>, for example:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\"><a href=\"javascript:void(0)\" data-symbol=\"MaxPlus.Core.EvalMAXScript\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;py_ref/class_max_plus_1_1_core.html#a00c76297f347dd3d50c629827c7217a1&quot;,&quot;title&quot;:&quot;MaxPlus Python API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">MaxPlus.Core.EvalMAXScript</a>(&quot;fn b2a b = (return b as Array)&quot;)\n\
rt.b2a(selectedFaces)\n\
</pre></div><div class=\'section\'><a id=\"getting-the-scene-root-node\"></a><h2 id=\"getting-the-scene-root-node\">Getting The Scene Root Node</h2></div>\n\
<p>MAXScript (and therefore pymxs) does not have an equivalent of <span class=\'code\'>MaxPlus.Core.GetRootNode()</span>, but the functionality can be easily implemented. Here is an example that gets the root node and traverses its children:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">from pymxs import runtime as rt\n\
\n\
def getRootNode():\n\
    rootScene = rt.rootScene\n\
    worldSubAnim = rootScene[rt.Name(&#39;world&#39;)]\n\
    return worldSubAnim.object\n\
    \n\
def outputNode(n, indent = &#39;&#39;):\n\
    print indent, n.Name\n\
    for c in n.Children:\n\
        outputNode(c, indent + &#39;--&#39;)\n\
        \n\
if __name__ == &#39;__main__&#39;:\n\
    outputNode(getRootNode())\n\
</pre></div><div class=\'section\'><a id=\"registering-python-callbacks\"></a><h2 id=\"registering-python-callbacks\">Registering Python Callbacks</h2></div>\n\
<p>Some MAXScript callback registration functions can take a Python function as the callback argument. These include <span class=\'code\'>registerTimeCallback()</span>, <span class=\'code\'>registerRedrawViewsCallback()</span>, and <span class=\'code\'>nodeEventCallback()</span>. Others take a string rather than a function as an argument, and thus cannot take a Python function. These functions include <span class=\'code\'>DialogMonitorOPS.RegisterNofication()</span> and <span class=\'code\'>callbacks.addScript()</span>.</p>\n\
<p>One work-around is to define a variable in the MAXScript runtime using <span class=\'code\'>pymxs.runtime.Execute()</span>, and then assign the Python function to the variable. Here is a simple example to illustrate the idea:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">import pymxs\n\
rt = pymxs.runtime\n\
\n\
def myCallback():\n\
    print &quot;Callback fired!&quot;\n\
\n\
# connect to the MXS runtime\n\
rt.Execute (&#39;pyCallback = undefined&#39;)\n\
rt.pyCallback = myCallback\n\
\n\
# register ourselves as a callback\n\
rt.callbacks.addScript(rt.Name(&#39;nodeCreated&#39;), &#39;pyCallback()&#39;)\n\
</pre></div>      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";