var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head><script src=\"../../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"></script><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>MAXScript Values in pymxs</title>\n\
   <meta name=\"topic-subtype\" content=\"Python\"></head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style><script>$(document).ready(function() { yepnope.injectJs(\"./scripts/multireflink.js\"); });</script><script></script><div id=\"reflinkdiv\"></div>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>MAXScript Values in pymxs</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"maxscript-values-in-pymxs\"></a></div>\n\
<p>This topic covers how Python and MAXScript values and types interoperate with pymxs.</p>\n\
<div class=\'section\'><a id=\"maxscript-type-marshalling\"></a><h2 id=\"maxscript-type-marshalling\">MAXScript Type Marshalling</h2></div>\n\
<p>For simple types such as integers, floats and strings, <span class=\'code\'>pymxs</span> has a direct reference to the data, and these values can be obtained and set directly.</p>\n\
<p>For complex types such as Array, pymxs wraps these objects with a <span class=\'code\'>MXSWrapper</span> object, <span class=\'code\'>pymxs.MXSWrapperBase</span>. For example:</p>\n\
<div class=\'codeBlock\'><pre>import pymxs\n\
rt = pymxs.runtime\n\
\n\
# Create and visit a MaxScript Array\n\
rt.execute(&quot; gma = #(1,2,3)&quot;)\n\
print rt.gma    #output: #(1,2,3)\n\
len(rt.gma)    #output: 3\n\
rt.gma[1] = 999\n\
rt.Print(rt.gma[1])    #output: 999\n\
l = list(rt.gma) #[1,999,3]\n\
l[2] = 888\n\
print l    #output: [1, 888, 3]\n\
print rt.gma   # output: #(1, 999, 3)\n\
\n\
# Create a MaxScript Array via python sequence\n\
ma = rt.Array(*(1, 2, 3))\n\
print ma    #output: #(1,2,3)\n\
</pre></div><p>For a demonstration of how to create, manipulate and cast complex MAXScript types in Python, see the <em>demoPyMXSTypeInterop.py</em> example.</p>\n\
<div class=\'section\'><a id=\"maxscript-names\"></a><h2 id=\"maxscript-names\">MAXScript Names</h2></div>\n\
<p>Name literals are commonly used in MAXScript to signify options passed to functions, and have no corresponding type in Python. They can be created with <span class=\'code\'>pymxs.runtime.Name()</span>. For example, to use the equivalent of <span class=\'code\'>toolmode.coordsys #world</span>:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">pymxs.runtime.toolMode.coordsys(pymxs.runtime.Name(&quot;world&quot;))\n\
</pre></div><p>Or to get all the properties for a box, using the equivalent of <span class=\'code\'>getpropnames box</span>:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">boxprops = pymxs.runtime.getpropnames(pymxs.runtime.box)\n\
</pre></div><p>This yields a list containing runtime.Name objects, however. It might be more useful to generate a list of Python strings:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">boxprops = [str(x) for x in list(pymxs.runtime.getpropnames(pymxs.runtime.box))]\n\
# gives: [&#39;typeinCreationMethod&#39;, &#39;typeInPos&#39;, &#39;typeInLength&#39;, &#39;typeInWidth&#39;, &#39;typeInHeight&#39;, &#39;length&#39;, &#39;width&#39;, &#39;height&#39;, &#39;widthsegs&#39;, &#39;lengthsegs&#39;, &#39;heightsegs&#39;, &#39;mapcoords&#39;]\n\
</pre></div><div class=\'section\'><a id=\"by-reference-parameter-handling\"></a><h2 id=\"by-reference-parameter-handling\">By-Reference Parameter Handling</h2></div>\n\
<p>Values passed back to Python from pymxs MAXScript functions that take a by-reference parameter, can be translated to Python values using the <span class=\'code\'>pymxs.mxsreference()</span> function. Note that this is supported for types other than Python lists.</p>\n\
<p>For example:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">import MaxPlus\n\
rt = pymxs.runtime\n\
\n\
<a href=\"javascript:void(0)\" data-symbol=\"MaxPlus.Core.EvalMAXScript\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;py_ref/class_max_plus_1_1_core.html#a00c76297f347dd3d50c629827c7217a1&quot;,&quot;title&quot;:&quot;MaxPlus Python API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">MaxPlus.Core.EvalMAXScript</a>(&quot;mySphere = sphere radius:10&quot;)\n\
<a href=\"javascript:void(0)\" data-symbol=\"MaxPlus.Core.EvalMAXScript\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;py_ref/class_max_plus_1_1_core.html#a00c76297f347dd3d50c629827c7217a1&quot;,&quot;title&quot;:&quot;MaxPlus Python API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">MaxPlus.Core.EvalMAXScript</a>(&quot;myType = #reference&quot;)\n\
<a href=\"javascript:void(0)\" data-symbol=\"MaxPlus.Core.EvalMAXScript\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;py_ref/class_max_plus_1_1_core.html#a00c76297f347dd3d50c629827c7217a1&quot;,&quot;title&quot;:&quot;MaxPlus Python API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">MaxPlus.Core.EvalMAXScript</a>(&quot;myResultArray = Array()&quot;)\n\
\n\
offset1 = rt.Point3(50.0, 50.0, 10.0)\n\
myReturnedList = []\n\
rt.maxOps.cloneNodes(rt.mySphere,  offset=offset1, cloneType=rt.myType, newNodes=pymxs.mxsreference(myReturnedList))\n\
print &quot;python array: &quot;\n\
print myReturnedList\n\
\n\
offset2 = rt.Point3(-50.0, -50.0, 10.0)\n\
rt.maxOps.cloneNodes(rt.mySphere,  offset=offset2, cloneType=rt.myType, newNodes=pymxs.mxsreference(rt.myResultArray))\n\
print &quot;runtime array: &quot;\n\
print rt.myResultArray\n\
\n\
</pre></div><p><strong>Note:</strong> for variables other than Python lists, the variable must be visible in the <span class=\'code\'>pymxs.runtime</span> context. For example, to pass a variable of type Color:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">import pymxs\n\
rt = pymxs.runtime\n\
<a href=\"javascript:void(0)\" data-symbol=\"MaxPlus.Core.EvalMAXScript\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;py_ref/class_max_plus_1_1_core.html#a00c76297f347dd3d50c629827c7217a1&quot;,&quot;title&quot;:&quot;MaxPlus Python API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../../&quot;}]\">MaxPlus.Core.EvalMAXScript</a>(&quot;myColor = color 100 10 10&quot;)\n\
print rt.myColor\n\
rt.maxOps.colorById(2,pymxs.mxsreference(rt.myColor))\n\
print rt.myColor\n\
</pre></div><p>Whereas, this approach will not work:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">import pymxs\n\
rt = pymxs.runtime\n\
myColor=rt.color(10,10,10)\n\
print myColor\n\
rt.maxOps.colorById(2,pymxs.mxsreference(myColor))\n\
print myColor\n\
</pre></div><div class=\'section\'><a id=\"dictionaries\"></a><h2 id=\"dictionaries\">Dictionaries</h2></div>\n\
<p>MAXScript introduced the Dictionary type in 3ds Max 2017.1, and by default use name literals for key values.</p>\n\
<p>Consider this MAXScript dictionary:</p>\n\
<div class=\'codeBlock\'><pre>my_dict = Dictionary #(#one, 1) #(#two, 2)\n\
</pre></div><p>In pymxs, this value is wrapped, and can be used like a MAXScript dictionary, for example getting the keys property:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">&gt;&gt;&gt; my_py_dict = pymxs.runtime.my_dict\n\
&lt;Dictionary&lt;Dictionary #name one:1 two:2 &gt;&gt;\n\
&gt;&gt;&gt; my_py_dict.keys\n\
&lt;Array&lt;#(#one, #two)&gt;&gt;\n\
</pre></div><p>In some cases, it may be more convenient to convert the MAXScript Dictionary to a native Python Dict:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">my_py_dict = {str(key):myd[key] for key in pymxs.runtime.my_dict.keys}\n\
# yields: {&#39;two&#39;: 2, &#39;one&#39;: 1}\n\
</pre></div><div class=\'section\'><a id=\"using-pymxs-objects-as-python-dict-keys\"></a><h2 id=\"using-pymxs-objects-as-python-dict-keys\">Using pymxs objects as Python Dict keys</h2></div>\n\
<p>As of 3ds Max 2019 Update 1, you can use some pymxs objects (such as scene nodes) as key values in native Python dictionaries (though not in MAXScript Dictionaries). For example:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\"> import pymxs\n\
rt = pymxs.runtime\n\
\n\
t1 = rt.teapot()\n\
t2 = rt.teapot(pos=rt.point3(20,20,0))\n\
t2.parent = t1\n\
\n\
node_dict = {t1:&quot;Teapot 1&quot;, t2:&quot;Teapot2&quot;}\n\
\n\
print (node_dict.keys())\n\
print (node_dict.values())\n\
</pre></div><p>To determine whether a MAXScript object can be used as a Python dictionary key, you can check whether it implements the python <span class=\'code\'>__hash__()</span> function.</p>\n\
      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";