var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head><script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"></script><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Using PySide2</title>\n\
   <meta name=\"topic-subtype\" content=\"Python\"></head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style><script>$(document).ready(function() { yepnope.injectJs(\"./scripts/multireflink.js\"); });</script><script></script><div id=\"reflinkdiv\"></div>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Using PySide2</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"using-pyside2\"></a></div>\n\
<p>Autodesk 3ds Max ships with a pre-built version of PySide 2.0 compatible with Python 2.7.15. This version includes all standard PySide modules.</p>\n\
<div class=\'section\'><a id=\"example-pyside-script\"></a><h2 id=\"example-pyside-script\">Example PySide Script</h2></div>\n\
<p>The following simple example shows how to obtain a handle for the PySide application object and create a dockable widget:</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">&#39;&#39;&#39;\n\
Demonstrates how to ceate a QDockWidget with PySide2 for use in 3ds Max\n\
&#39;&#39;&#39;\n\
\n\
from PySide2 import QtCore\n\
from PySide2 import QtGui\n\
from PySide2 import QtWidgets\n\
import pymxs\n\
\n\
def make_cylinder():\n\
    cyl = pymxs.runtime.Cylinder(radius=10, height=30)\n\
    pymxs.runtime.redrawViews()\n\
    return    \n\
    \n\
class PyMaxDockWidget(QtWidgets.QDockWidget):\n\
    def __init__(self, parent=None):\n\
        super(PyMaxDockWidget, self).__init__(parent)\n\
        self.setWindowFlags(QtCore.Qt.Tool)\n\
        self.setWindowTitle(&#39;Pyside Qt  Dock Window&#39;)\n\
        self.initUI()\n\
        self.setAttribute(QtCore.Qt.WA_DeleteOnClose)\n\
        \n\
    def initUI(self):\n\
        main_layout = QtWidgets.QVBoxLayout()\n\
        label = QtWidgets.QLabel(&quot;Click button to create a cylinder in the scene&quot;)\n\
        main_layout.addWidget(label)\n\
\n\
        cylinder_btn = QtWidgets.QPushButton(&quot;Cylinder&quot;)\n\
        cylinder_btn.clicked.connect(make_cylinder)\n\
        main_layout.addWidget(cylinder_btn)\n\
        widget = QtWidgets.QWidget()\n\
        widget.setLayout(main_layout)\n\
        self.setWidget(widget)\n\
        self.resize(250, 100)    \n\
\n\
def main():\n\
    pymxs.runtime.resetMaxFile(pymxs.runtime.name(&#39;noPrompt&#39;))\n\
    mainWindow=QtWidgets.QWidget.find(pymxs.runtime.windows.getMAXHWND())\n\
    w = PyMaxDockWidget(parent=mainWindow)\n\
    w.setFloating(True)\n\
    w.show()\n\
\n\
if __name__ == &#39;__main__&#39;:\n\
    main()\n\
</pre></div><p><em>Notes</em>:</p>\n\
<ul>\n\
<li>To make 3ds Max the parent of our <span class=\'code\'>QWidget</span>, we parent it to the main window object found from the window handled returned by <span class=\'code\'>pymxs.runtime.windows.getMAXHWND()</span>. (You can also use <span class=\'code\'>MaxPlus.GetQMaxMainWindow()</span>).</li>\n\
<li>Objects created in your script need to be protected from being garbage collected. There are two approaches you can take. One is to keep references to them in a static class variable. The other is to inherit from a <span class=\'code\'>QWidget</span> class (in this example, <span class=\'code\'>QDockWidget</span>), which will hold the static reference.</li>\n\
<li>Nodes created within a script may not appear in Max until the script exits, unless you force a viewport update. You can do so with <span class=\'code\'>pymxs.runtime.redrawViews()</span>, <span class=\'code\'>MaxPlus.ViewportManager.ForceCompleteRedraw()</span> or <span class=\'code\'>MaxPlus.ViewportManager.RedrawViews()</span>.</li>\n\
<li>A non-docking UI can derive from <span class=\'code\'>QWidget</span>. To display the widget as modeless, call <span class=\'code\'>.show()</span>, and to open it as modal, call <span class=\'code\'>.exec_()</span>.</li>\n\
<li>Though not strictly necessary, this example sets the window flags to <span class=\'code\'>QtCore.Qt.Tool</span> to improve the appearance of the dialog, and sets the attribute <span class=\'code\'>QtCore.Qt.QA_DeleteOnClose</span> to delete the dialog immediately on close.</li>\n\
</ul>\n\
<div class=\'section\'><a id=\"loading-qt-ui-files\"></a><h2 id=\"loading-qt-ui-files\">Loading Qt UI Files</h2></div>\n\
<p>MaxPlus provides a facility for loading UI files created by Qt Designer: <span class=\'code\'>MaxPlus.LoadUiType()</span>. This example looks for a ui file named <span class=\'code\'>test.ui</span> located under <span class=\'code\'>scripts/Python/ui/</span> and loads it.</p>\n\
<p><strong>Note</strong>: If you use the Python global <span class=\'code\'>__file__</span> to find the target ui file, the global will only evaluate when using the <strong>Scripting &gt; Run Script</strong> command. In this example, we use the MAXScript <span class=\'code\'>getDir()</span> function, which will work in any context.</p>\n\
<div class=\'codeBlock\'><pre class=\"prettyprint lang-py\">import MaxPlus, os\n\
from PySide2 import QtWidgets\n\
from PySide2 import QtCore\n\
from pymxs import runtime as rt\n\
\n\
def makeTeapot():\n\
    rt.teapot()\n\
    \n\
class TestDialog(QtWidgets.QDialog):\n\
    def __init__(self, ui_class, parent=MaxPlus.GetQMaxMainWindow()):\n\
        QtWidgets.QDialog.__init__(self, parent)\n\
        self.ui=ui_class()\n\
        self.ui.setupUi(self)\n\
        self.ui.pushButton.clicked.connect(makeTeapot)\n\
\n\
def main():\n\
    script_dir = rt.getDir(rt.name(&#39;scripts&#39;))  \n\
    ui_path = os.path.join(script_dir, &#39;Python&#39;, &#39;ui&#39;,&#39;test.ui&#39;)\n\
    ui_class, base_class = MaxPlus.LoadUiType(ui_path)\n\
    instance = TestDialog(ui_class)\n\
    instance.show()\n\
\n\
\n\
if __name__ == &#39;__main__&#39;:\n\
    main()\n\
</pre></div>      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";